<?php

namespace Strig\Category\Adapter;

use Strig\Category\CategoryUrlGeneratorInterface;
use Strig\Category\Entity\Category;
use Strig\Gath\Gath;
use Zend\Hydrator\NamingStrategy\MapNamingStrategy;
use Zend\Hydrator\Reflection;
use Zend\Hydrator\Strategy\ClosureStrategy;

class GathCategoryAdapter implements CategoryAdapterInterface
{
    protected $gath;
    protected $urlGenerator;

    public function __construct(Gath $gath, CategoryUrlGeneratorInterface $urlGenerator)
    {
        $this->gath = $gath;
        $this->urlGenerator = $urlGenerator;
    }

    public function getAll()
    {
        return $this->get();
    }

    public function getByParent($parent)
    {
        return $this->get($parent);
    }

    protected function get($parent = 'ROOT')
    {
        $data = $this->gath->getCategories($parent, 5);
        $data = $this->chopTree($data['Results']);


        $namingStrategyArray = [
            'Code' => 'id',
            'Name' => 'name',
            'ParentCode' => 'parentId',
            'Menu_order' => 'order',
            '_sale_badge_text' => 'saleBadgeText',
            '_sale_badge_size' => 'saleBadgeSize',
            '_upsale_tecdoc_artid' => 'upsaleTecdocArtId',
            '_upsale_artcodes' => 'upsaleArtCodes'
        ];

        $hydrator = new Reflection();
        $hydrator->setNamingStrategy(new MapNamingStrategy($namingStrategyArray));

        try {
            $unescapeStrategy = new ClosureStrategy(null, function ($value) {
                return html_entity_decode($value);
            });

            $customExplodeStrategy = new ClosureStrategy(null, function ($value) {
                return array_map('intval', array_filter(explode(';', $value)));
            });

            $castIntStrategy = new ClosureStrategy(null, function ($value) {
                if ($value === 'ROOT') return $value;
                return (int)$value;
            });

            $hydrator->addStrategy('id', $castIntStrategy);
            $hydrator->addStrategy('parentId', $castIntStrategy);
            $hydrator->addStrategy('order', $castIntStrategy);

            $hydrator->addStrategy('name', $unescapeStrategy);
            $hydrator->addStrategy('description', $unescapeStrategy);

            $hydrator->addStrategy('genericArticleIds', $customExplodeStrategy);
            $hydrator->addStrategy('genericArticleMotoIds', $customExplodeStrategy);
            $hydrator->addStrategy('upsaleTecdocArtId', $customExplodeStrategy);
            $hydrator->addStrategy('upsaleArtCodes', $customExplodeStrategy);
        } catch (\Exception $e) {
        }

        $extractor = new Reflection();

        $tree = [];
        $generator = $this->urlGenerator;
        foreach ($data as $datum) {
            /** @var Category $category */
            $category = $hydrator->hydrate((array)$datum, new Category());
            $category->setUrl($generator($category));
            $tree[] = $extractor->extract($category);
        }


        return $tree;
    }

    /**
     * @param array $categories
     * @param array $dimensions
     * @param string $path
     * @param string $baseUri
     */
    public function getSprites(array $categories, array $dimensions, string $path, string $baseUri)
    {
        // TODO: Implement getSprites() method.
    }

    /**
     * @param array $categories
     * @param string $dimension
     * @param string $path
     * @param string $baseUri
     */
    public function getImages(array $categories, string $dimension, string $path, string $baseUri)
    {
        // TODO: Implement getImages() method.
    }


    protected function chopTree(array $tree, string $children = 'Children')
    {
        $flat = [];

        foreach ($tree as $key => &$value) {
            // haxx, za redoslijed
            $flat[] = array_filter($value, function($key) use ($children) {
                return $key !== $children;
            }, ARRAY_FILTER_USE_KEY);;

            if (array_key_exists($children, $value)) {
                $flat = array_merge($flat, $this->chopTree($value[$children]));
            }
        }

        return $flat;
    }

}
