<?php

namespace Strig\Category;

use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Strig\Cache;
use Strig\Category\Middleware\CategoryByParentMiddleware;
use Strig\Category\Middleware\CategoryImagesMiddleware;
use Strig\Category\Middleware\CategorySpritesMiddleware;
use Strig\Category\Repository\CategoryRepository;
use Zend\Expressive\Middleware\LazyLoadingMiddleware;
use Zend\Expressive\MiddlewareContainer;
use Zend\Stratigility\MiddlewarePipe;
use function Zend\Stratigility\path;

class CategoryMiddleware implements MiddlewareInterface
{

    protected $pipeline;

    public function __construct(ContainerInterface $container, MiddlewareContainer $middlewareContainer)
    {
        $this->pipeline = new MiddlewarePipe();

        /** @var CategoryManager $manager */
//        $manager = $container->get(CategoryManager::class);

        /** @var CategoryRepository $repository */
//        $repository = $manager->getRepository(Category::class);

        $this->pipeline->pipe(path('/sprites', new LazyLoadingMiddleware($middlewareContainer, CategorySpritesMiddleware::class)));
        $this->pipeline->pipe(path('/images', new LazyLoadingMiddleware($middlewareContainer, CategoryImagesMiddleware::class)));


        $this->pipeline->pipe(path('/', new LazyLoadingMiddleware($middlewareContainer, Cache\Middleware::class)));
        $this->pipeline->pipe(path('/', new LazyLoadingMiddleware($middlewareContainer, CategoryByParentMiddleware::class)));
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {

        return $this->pipeline->process($request, $handler);
    }
}
