<?php

namespace Strig\Category\Handler;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Strig\Category\Repository\CategoryRepository;
use Strig\Category\RestDispatch;
use Zend\Diactoros\Response\JsonResponse;

class CategoryHandler implements RequestHandlerInterface
{
    use RestDispatch;

    protected $repository;

    public function __construct(CategoryRepository $categoryRepository)
    {

        $this->repository = $categoryRepository;
    }


    /**
     * Handle the request and return a response.
     * @param ServerRequestInterface $request
     * @return ResponseInterface
     */
    public function get(ServerRequestInterface $request): ResponseInterface
    {
        $parentId = $request->getAttribute('parentId');

        if ($parentId) {
            return $this->getByParentId($request, $parentId);
        }

        $data = $this->repository->getAll();

        return new JsonResponse($data);
    }


    /**
     * @param ServerRequestInterface $request
     * @param int $parent
     * @return ResponseInterface
     */
    public function getByParentId(ServerRequestInterface $request, int $parent): ResponseInterface
    {
        $data = $this->repository->getByParent($parent);

        return new JsonResponse($data);
    }
}
