<?php

namespace Strig\Category\Middleware;

use Ep\Middleware\BaseUrl;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Strig\Category\Repository\CategoryRepository;
use Zend\Diactoros\Response\JsonResponse;

class CategoryImagesMiddleware implements MiddlewareInterface
{

    protected $repository;

    public function __construct(CategoryRepository $categoryRepository)
    {
        $this->repository = $categoryRepository;
    }


    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $baseUri = (string) $request->getAttribute(BaseUrl::BASE_URL);
        $uri = $request->getUri();
        $path = substr($uri->getPath(), 1);

        $pieces = explode('/', $path);

        list($categories, $dimension) = $pieces;
        $categories = explode(',', $categories);

        $data = $this->repository->getImages($categories, $dimension, $path, $baseUri);

        return new JsonResponse($data);
    }
}
